%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Initialize the program                        %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all
% close all
clc
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Enter the transfer function of digital system %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
num=[0.06 0.034 0.071];                 %numerator
den=[1 -0.606 -0.747 0.519 0 0 0 0];    %denominator
Ts=0.01;                                %sampling time
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Normalize the transfer function               %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
K=num(1)/den(1);                        %static gain
num_Gfb=num/num(1);                     %numerator
den_Gfb=den/den(1);                     %denominator
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Now we will find the non-minimum phase zeros  %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
roots_numGfb=roots(num_Gfb)';
roots_denGfb=roots(den_Gfb)';

num_mp=1;       num_nmp=1;
roots_mp=[];    roots_nmp=[];
mp=0;           nmp=0;

for i=1:(length(num)-1)
    if abs(roots_numGfb(i))>1
        nmp=nmp+1;
        roots_nmp(nmp)=roots_numGfb(i);
        num_nmp=conv(num_nmp,[1 -roots_numGfb(i)]);
    else
        mp=mp+1;
        roots_mp(mp)=roots_numGfb(i);
        num_mp=conv(num_mp,[1 -roots_numGfb(i)]);
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Now we will find the real and complex         %%% 
%%% non-minimum phase zeros                       %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
roots_nmp_re = [];      roots_nmp_im = []; 
nmp_re=0;               nmp_im=0;

if isempty(roots_nmp) ~= 1
    for i=1:nmp
        if imag(roots_nmp(i)) == 0
            nmp_re = nmp_re + 1;
            roots_nmp_re(nmp_re) = roots_nmp(i);
        else
            nmp_im = nmp_im + 1;
            roots_nmp_im(nmp_im) = roots_nmp(i);
        end       
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Choose the desired bandwidth of feedforward   %%%
%%% compensator                                   %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
theta=linspace(0,pi/2,1000);
z = exp(j*theta);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Choose the amount of non-minimum phase zeros  %%%
%%% (real and complex)                            %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sol_re = 2;
sol_im = 0;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Choose a weighting function                   %%%
%%% (real and complex)                            %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
num_wth = [3.61];
den_wth = conv([1 0.9],[1 0.9]);

Wth = polyval(num_wth,z)./polyval(den_wth,z);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%yapilacak optimizasyon isleminde kullanilacak parametre degerleri
m=10;                       %paralel islenecek zm aday kmesi.
n=sol_re+sol_im;			%parametre sayisi veya zm aday vektrlerinin boyutu.
stopcr=1000; 				%durma kriteri. Aramayi en fazla ka adim yapacagini belirler..
maxdenx=10;                 %10 bagimsiz deneme yapacak.
nbit=16;					%parametredeki bit sayisi.
maxval=1;                   %her parametrenin alacagi maksimum deger.
minval=-1;                  %her parametrenin alacagi minimum deger.

kusa=2^nbit;				%kuanta sayisi
scl=(maxval-minval)/kusa;			%normalizasyon parametresi

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Big Bang Big Crunch Optimization Routine      %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%evrimsel algoritmada kullanilacak ilk nesil programa giriliyor.
ilkbirey = kusa.*[rand(m,sol_re) rand(m,2*sol_im)];

%Sphere fonksiyonu iin iyilesme grafigindeki referans iin cost0 hesaplanir.
for birey_no=1:m,
    cost0(birey_no)=MagCost2(K,num_nmp,roots_nmp_re,roots_nmp_im,sol_re,sol_im,minval+scl.*ilkbirey(birey_no,:),z,Wth);
    cost0(birey_no)=abs(cost0(birey_no));
end

[eniyi,j] = min(cost0);		%butun denemelerdeki eniyiyi tutmak icin burada resetlendi...
bireyopt = ilkbirey(j,:);
%-------------------------

for deneme=1:maxdenx,
    
eniyi=min(cost0);		%referans eniyi degeri
birey=ilkbirey;
iterval(deneme)=0;
tic                         %Zamanlayiciyi baslat
%*******************************************************************
for itersa=1:stopcr,		%Dnglerin baslangici

	%***************cost function degerlerinin bulunmasi**************
   sumcosts=0;
   for birey_no=1:m,
      costs(birey_no)=MagCost2(K,num_nmp,roots_nmp_re,roots_nmp_im,sol_re,sol_im,minval+scl.*birey(birey_no,:),z,Wth);
      costs(birey_no)=abs(costs(birey_no));
      if costs(birey_no)<eniyi
         eniyi=costs(birey_no);
         bireyopt=birey(birey_no,:);
      end
      sumcosts = sumcosts + 1/(costs(birey_no)+0.0001);  	
   end

   % Yaklasim islemi
   for parametre_no=1:n,
      ortanokta(parametre_no)=0;
      for birey_no=1:m,
         ortanokta(parametre_no)=ortanokta(parametre_no)+birey(birey_no,parametre_no)*(1/(costs(birey_no)+0.0001));	
      end
      ortanokta(parametre_no)=floor((1/sumcosts)*ortanokta(parametre_no));
   end

   % Dagitim islemi (ortanokta etrafinda normal dagilmis yeni noktalar turetir)
   for birey_no=1:m-1,
      for parametre_no=1:n,
%         sapma=floor((kusa-1)*randn(1)*(1/itersa));
         sapma=floor((kusa-1)*randn(1));
         yeni_parametre=ortanokta(parametre_no)+sapma;
         if yeni_parametre>=kusa
            yeni_parametre = kusa-1;
         end
         if yeni_parametre<0
            yeni_parametre = 0;
         end
         birey(birey_no,parametre_no) = yeni_parametre;
      end
   end
sonuc(deneme,itersa)=eniyi;
end							%Dongulerin sonu
tspan(deneme)=toc;          %Zamanlayiciyi durdur
end							%Bagimsiz denemelerin sonu

%raporlama
% [sonuc,iterval,tspan]=chauvenet(sonuc,iterval,tspan); %Chauvenet kriterini uygulayarak sonucu duzeltiyoruz. 
ortalama = mean(sonuc);
% stdspm   = std(sonuc(:,length(sonuc)))
% CPUtime  = mean(tspan)
% funceval = mean(iterval)
figure(2)
plot(ortalama)
xlabel('iteration')
ylabel('fitness value')
cozum_yeni = minval+scl.*bireyopt
%eniyi

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Choose the desired bandwidth of feedforward   %%%
%%% compensator                                   %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
