
function [COMES_wrkspc] = PrmSpcCalc(COMES_wrkspc)


% Parametres
% -------------------------------------------------------------------------
sigma=COMES_wrkspc.rbstctrl.DStability.Constraints.sigma;
alpha=COMES_wrkspc.rbstctrl.DStability.Constraints.alpha*pi/180;
R=COMES_wrkspc.rbstctrl.DStability.Constraints.R;

% Transferfunction of plant
num_p=COMES_wrkspc.System.Model.numg;
den_p=COMES_wrkspc.System.Model.deng;

% Transfer function of controller
num_c=subs(COMES_wrkspc.rbstctrl.Controller.Model.num,{COMES_wrkspc.rbstctrl.Controller.Label.x1,COMES_wrkspc.rbstctrl.Controller.Label.x2},{sym('x1','real'),sym('x2','real')});
den_c=subs(COMES_wrkspc.rbstctrl.Controller.Model.den,{COMES_wrkspc.rbstctrl.Controller.Label.x1,COMES_wrkspc.rbstctrl.Controller.Label.x2},{sym('x1','real'),sym('x2','real')});

syms s w r theta x1 x2 real

G=poly2sym(num_p,s)/poly2sym(den_p,s);

C=poly2sym(num_c,s)/poly2sym(den_c,s);

L=C*G;

T=simplify(L/(1+L));

[N,D]=numden(T);

% -------------------------------------------------------------------------
% Hurwitz Stability
if COMES_wrkspc.rbstctrl.DStability.Options.Hurwitz.Index == 1
    
    d=subs(D,'s',j*w);
    
    eqn1=real(d);
    
    eqn2=simplify(imag(d));
    
    sol1=solve(eqn1, eqn2, 'x1','x2');
    
    COMES_wrkspc.rbstctrl.DStability.Solutions.sol1.x1 = subs(sol1.x1,'w',linspace(COMES_wrkspc.rbstctrl.DStability.Options.Hurwitz.StrFrq,COMES_wrkspc.rbstctrl.DStability.Options.Hurwitz.EndFrq,COMES_wrkspc.rbstctrl.DStability.Options.Hurwitz.GridFrq));
    
    COMES_wrkspc.rbstctrl.DStability.Solutions.sol1.x2 = subs(sol1.x2,'w',linspace(COMES_wrkspc.rbstctrl.DStability.Options.Hurwitz.StrFrq,COMES_wrkspc.rbstctrl.DStability.Options.Hurwitz.EndFrq,COMES_wrkspc.rbstctrl.DStability.Options.Hurwitz.GridFrq));
    
end

% % real Root Boundary
% subs(D,'s',0)

% % Imaginar Root Boundary
% subs(expand(D/s^4),'s',inf)
    
% -------------------------------------------------------------------------
% Settling Time Constraint
if COMES_wrkspc.rbstctrl.DStability.Options.SettlingTime.Index == 1
    
    d=subs(D,'s',-sigma+j*w);

    eqn1=real(d);

    eqn2=simplify(imag(d));

    sol2=solve(eqn1, eqn2, 'x1','x2');
    
    COMES_wrkspc.rbstctrl.DStability.Solutions.sol2.x1 = subs(sol2.x1,'w',linspace(0,tan(pi-alpha)*sigma,COMES_wrkspc.rbstctrl.DStability.Options.SettlingTime.GridFrq));
    
    COMES_wrkspc.rbstctrl.DStability.Solutions.sol2.x2 = subs(sol2.x2,'w',linspace(0,tan(pi-alpha)*sigma,COMES_wrkspc.rbstctrl.DStability.Options.SettlingTime.GridFrq));

end

% -------------------------------------------------------------------------
% Damping Constraint
if COMES_wrkspc.rbstctrl.DStability.Options.Damping.Index == 1
    
    d=subs(D,'s',r*exp(j*alpha));

    eqn1=real(d);

    eqn2=simplify(imag(d));

    sol3=solve(eqn1, eqn2, 'x1','x2');
    
    COMES_wrkspc.rbstctrl.DStability.Solutions.sol3.x1 = subs(sol3.x1,'r',linspace(sigma/cos(pi-alpha),R,COMES_wrkspc.rbstctrl.DStability.Options.Damping.GridFrq));
    
    COMES_wrkspc.rbstctrl.DStability.Solutions.sol3.x2 = subs(sol3.x2,'r',linspace(sigma/cos(pi-alpha),R,COMES_wrkspc.rbstctrl.DStability.Options.Damping.GridFrq));

end

% -------------------------------------------------------------------------
% Bandwidth Constraint
if COMES_wrkspc.rbstctrl.DStability.Options.Bandwidth.Index == 1
    
    d=subs(D,'s',R*exp(j*theta));

    eqn1=real(d);

    eqn2=simplify(imag(d));

    sol4=solve(eqn1, eqn2, 'x1','x2');
    
    COMES_wrkspc.rbstctrl.DStability.Solutions.sol4.x1 = subs(sol4.x1,'theta',linspace(alpha,pi,COMES_wrkspc.rbstctrl.DStability.Options.Bandwidth1.GridFrq));
    
    COMES_wrkspc.rbstctrl.DStability.Solutions.sol4.x2 = subs(sol4.x2,'theta',linspace(alpha,pi,COMES_wrkspc.rbstctrl.DStability.Options.Bandwidth1.GridFrq));

    d=subs(D,'s', -sigma);

    sol5=solve(d, 'x1');
    
    COMES_wrkspc.rbstctrl.DStability.Solutions.sol5.x1 = subs(sol5,'x2',linspace(COMES_wrkspc.rbstctrl.DStability.Options.Bandwidth2.StrPrm,COMES_wrkspc.rbstctrl.DStability.Options.Bandwidth2.EndPrm,COMES_wrkspc.rbstctrl.DStability.Options.Bandwidth2.GridPrm));
    
    COMES_wrkspc.rbstctrl.DStability.Solutions.sol5.x2 = linspace(COMES_wrkspc.rbstctrl.DStability.Options.Bandwidth3.StrPrm,COMES_wrkspc.rbstctrl.DStability.Options.Bandwidth3.EndPrm,COMES_wrkspc.rbstctrl.DStability.Options.Bandwidth3.GridPrm);

    d=subs(D,'s', -1.0*R);

    sol6=solve(d, 'x1');
    
    COMES_wrkspc.rbstctrl.DStability.Solutions.sol6.x1 = subs(sol6,'x2',linspace(COMES_wrkspc.rbstctrl.DStability.Options.Bandwidth3.StrPrm,COMES_wrkspc.rbstctrl.DStability.Options.Bandwidth3.EndPrm,COMES_wrkspc.rbstctrl.DStability.Options.Bandwidth3.GridPrm));
    
    COMES_wrkspc.rbstctrl.DStability.Solutions.sol6.x2 = linspace(COMES_wrkspc.rbstctrl.DStability.Options.Bandwidth3.StrPrm,COMES_wrkspc.rbstctrl.DStability.Options.Bandwidth3.EndPrm,COMES_wrkspc.rbstctrl.DStability.Options.Bandwidth3.GridPrm);

end

%End Of Function

